#!/usr/bin/ruby

require 'json'
require 'socket'
require 'date'
require 'active_support/core_ext/hash'
require 'etc'

nvidia_smi=`/usr/bin/nvidia-smi -q -x`

nvidia_smi_h = Hash.from_xml(nvidia_smi)

processes = {}

$host = Socket.gethostname

$time = Time.now.to_i

$output = []

pidtojob={}

Dir['/sys/fs/cgroup/cpu/pbspro.slice/*'].each do |dir|
  next if dir !~ /pbspro-/
  jobid = dir.gsub('\x5b', '[').gsub('\x5d', ']').gsub('\x2d', '-')
  jobid = jobid.sub(/.*pbspro-(.*)\.slice/, '\1')
  File.read("#{dir}/cgroup.procs").each_line do |task|
    pidtojob[task.chomp] = jobid
  end
end

timeused = {}

if nvidia_smi_h['nvidia_smi_log'] and nvidia_smi_h['nvidia_smi_log']['gpu']
  nvidia_smi_h['nvidia_smi_log']['gpu'].each do |gpu|
    gpu_util = gpu['utilization']['gpu_util'].to_i
    mem_av = gpu['fb_memory_usage']['total'].to_i * 1024*1024
    begin
      pid = gpu['processes']['process_info']['pid']
      mem_used = gpu['processes']['process_info']['used_memory']
    rescue
      pid = -1
      mem_used = 0
    end
    if pid != -1
      if mem_used =~ /MiB/
        mem_used = mem_used.to_i * 1024 * 1024
      elsif mem_used =~ /[kK]iB/
        mem_used = mem_used.to_i * 1024
      elsif mem_used =~ /GiB/
        mem_used = mem_used.to_i * 1024 * 1024 * 1024
      else
        mem_used = mem_used.to_i
      end
      begin
        cmd = File.readlink("/proc/#{pid}/exe")
        user = Etc.getpwuid(File.stat("/proc/#{pid}/exe").uid)['name']
        while timeused[$time] 
          $time+=1
        end
        timeused[$time] = 1
	if cmd =~ /^\/scratch/
          cmd = cmd.sub(/\/job_[^\/]*/, '')
	end        
        $output << { 'name' => 'gpustats', 'host' => $host, 'time' => $time, 'mem' => mem_used, 'gpuutil' => gpu_util, 'jobid' => pidtojob[pid], 'cmd' => cmd.sub(/.*\//, ''), 'cmdfull' => cmd, 'memutil' => (mem_used * 100) / mem_av, 'user' => user }
      rescue
      end
    end
  end
end

f = File.new("/var/tmp/gpustats.json.new", "w", 0644)
f.write(JSON.pretty_generate($output))
f.close

File.rename("/var/tmp/gpustats.json.new", "/var/tmp/gpustats.json")

